package org.processmining.causalactivitygraphcreator.creators.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.causalactivitygraph.models.impl.CausalActivityGraphFactory;
import org.processmining.causalactivitygraphcreator.creators.GraphCreator;
import org.processmining.causalactivitygraphcreator.creators.GraphCreatorParameters;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.framework.plugin.PluginContext;

public class GraphStrongestCreator implements GraphCreator {

	public final static String NAME = "Strongest I/O Values";

	private class Record {
		double value;
		XEventClass source;
		XEventClass target;
	}

	public CausalActivityGraph apply(PluginContext context, CausalActivityMatrix matrix,
			GraphCreatorParameters parameters) {
		CausalActivityGraph graph = CausalActivityGraphFactory.createCausalActivityGraph();
		List<XEventClass> activities = matrix.getActivities();
		graph.init(matrix.getLabel(), new HashSet<XEventClass>(activities));
		List<Record> records = new ArrayList<Record>();
		for (XEventClass source : activities) {
			for (XEventClass target : activities) {
				Record record = new Record();
				record.value = (matrix.getValue(source, target) + 1.0) / 2.0;
				record.source = source;
				record.target = target;
				records.add(record);
			}
		}
		Collections.sort(records, new Comparator<Record>() {

			public int compare(Record o1, Record o2) {
				return (o1.value > o2.value ? -1 : (o1.value < o2.value ? 1 : 0));
			}

		});
		Map<XEventClass, Double> sourceThresholds = new HashMap<XEventClass, Double>();
		Map<XEventClass, Double> targetThresholds = new HashMap<XEventClass, Double>();
		for (Record record : records) {
			if (sourceThresholds.keySet().contains(record.source)) {
				if (sourceThresholds.get(record.source) < record.value) {
					graph.setCausality(record.source, record.target, record.value);
				}
			} else {
				graph.setCausality(record.source, record.target, record.value);
				sourceThresholds.put(record.source, 1.0 /*Math.pow(record.value, 2.0)*/);
			}
			if (targetThresholds.keySet().contains(record.target)) {
				if (targetThresholds.get(record.target) < record.value) {
					graph.setCausality(record.source, record.target, record.value);
				}
			} else {
				graph.setCausality(record.source, record.target, record.value);
				targetThresholds.put(record.target, 1.0 /*Math.pow(record.value, 2.0)*/);
			}
		}
//		for (XEventClass source : activities) {
//			for (XEventClass middle : activities) {
//				double smValue = graph.getCausality(source, middle);
//				if (smValue > 0.0) {
//					for (XEventClass target : activities) {
//						double mtValue = graph.getCausality(middle, target);
//						if (mtValue > 0.0) {
//							double stValue = graph.getCausality(source, target);
//							if (0.0 < stValue && stValue < smValue * mtValue) {
//								graph.resetCausality(source, target);
//							}
//						}
//					}
//				}
//			}
//		}
		return graph;
	}

	public String getName() {
		return NAME;
	}

}
