package org.processmining.causalactivitygraphcreator.creators.impl;

import java.util.HashSet;
import java.util.List;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.causalactivitygraph.models.impl.CausalActivityGraphFactory;
import org.processmining.causalactivitygraphcreator.creators.GraphCreator;
import org.processmining.causalactivitygraphcreator.creators.GraphCreatorParameters;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.framework.plugin.PluginContext;

public class GraphStrongestEdgeCreator implements GraphCreator {

	public final static String NAME = "Strongest Edges";

	public CausalActivityGraph apply(PluginContext context, CausalActivityMatrix matrix,
			GraphCreatorParameters parameters) {
		CausalActivityGraph graph = CausalActivityGraphFactory.createCausalActivityGraph();
		List<XEventClass> activities = matrix.getActivities();
		graph.init(matrix.getLabel(), new HashSet<XEventClass>(activities));

		for (XEventClass source : activities) {
			for (XEventClass target : activities) {
				double value = matrix.getValue(source, target);
				double inverseValue = matrix.getValue(target, source);
				boolean ok = (value > 0.0);
				if (ok && value < 0.5) {
					for (XEventClass activity : activities) {
						if (matrix.getValue(target, source) >= Math.sqrt(value)) {
							ok = false;
						}
						if (matrix.getValue(source, activity) >= value && matrix.getValue(activity, target) >= value) {
							ok = false;
						}
						if (matrix.getValue(activity, source) >= value && matrix.getValue(activity, target) >= value) {
							ok = false;
						}
						if (matrix.getValue(source, activity) >= value && matrix.getValue(target, activity) >= value) {
							ok = false;
						}
					}
				}
				if (ok) {
					graph.setCausality(source, target, value);
				}
			}
		}
		return graph;
	}

	public String getName() {
		return NAME;
	}
}
